@@ -34,7 +34,6 @@ class Database:
 
             
             if price_adj == False:
-
                 ohlc_result = json.loads(json.loads(response.content).replace("'", "\"").replace('False','0').replace('True','1'))
             else:
 
@@ -128,6 +127,14 @@ class Database:
                     elif freq == '1D':
                         df = df[['date','open','high','low','close','volume']]
 
+            df['date'] = df['date'].astype(int)
+            if 'T' in freq:
+                df['time'] = df['time'].astype(int)
+            else:
+                df['time'] = 0
+
+            df = df[['date', 'time', 'open', 'high', 'low', 'close', 'volume']]
+
             return df
     
     def get_us_stock_ohlc(self, code, start_date, end_date):
@@ -209,6 +216,7 @@ class Database:
                 if year_month not in year_month_involved_list:
                     year_month_involved_list.append(year_month)
 
+            #if end_date >= int(expiry_date_dict[max(year_month_involved_list)].strftime('%Y%m%d')):
             year_month_involved_list.append((datetime.datetime.strptime(year_month_involved_list[-1], '%y%m') + relativedelta(months=1)).strftime('%y%m'))
 
             year_month_involved_list_str = json.dumps(str(year_month_involved_list))
@@ -232,11 +240,13 @@ class Database:
             link_str = link_url + index_str + '&' + freq_str + '&' + rolling_day_str + '&' + rolling_time_str + '&' + year_month_involved_list_str + '&' \
                        + token_str + '&' + database_str + '&' \
                        + start_date_str + '&' + end_date_str
-            
+
+
             response = requests.get(link_str)
             result_list = json.loads(json.loads(response.content).replace("'", "\""))
 
             df_list = []
+            date_time_list_list = []
 
             for result in result_list:
                 sub_df_list = []
@@ -250,7 +260,7 @@ class Database:
                     df = df.reset_index(drop=True)
                     temp_date_list = list(df['date'].unique())
                     temp_date_list.sort()
-                    
+
                     date_list = []
                     for temp_date in temp_date_list:
                         if datetime.datetime.strptime(str(temp_date),'%Y%m%d').weekday() < 5:
@@ -263,7 +273,7 @@ class Database:
 
                     for i in range(len(date_list)):
                         date_item = date_list[i]
-    
+
                         if i + rolling_day <= len(date_list) - 1:
                             if date_list[i + rolling_day] == front_expiry_date:
                                 front_cut_off_date = date_item
@@ -272,32 +282,43 @@ class Database:
                                 break
                         else:
                             back_cut_off_date = max(date_list)
-                                        
+
                     if df.loc[0,'expiry_date'] == '20130530':
                         front_cut_off_date = 20130501
-                    
-                    if back_cut_off_date != end_date:
+
+                    if (back_cut_off_date != end_date) or (back_cut_off_date == end_date and back_cut_off_date not in expiry_date_list):
                         df = df[(df['date'] > front_cut_off_date) | ((df['date'] == front_cut_off_date) & (df['time'] >= rolling_time))]
                         df = df[(df['date'] < back_cut_off_date) | ((df['date'] == back_cut_off_date) & (df['time'] < rolling_time))]
-                    
-                    df_list.append(df)
-    
+
+                    df = df[df['date'] >= start_date]
+                    df = df[df['date'] <= end_date]
+                    df['date'] = df['date'].astype(str)
+                    df['time'] = df['time'].astype(str)
+                    df['time'] = df['time'].str.zfill(6)
+                    df['datetime'] = df['date'] + ' ' + df['time']
+
+                    df = df.reset_index(drop=True)
+                    date_time_list = df['datetime'].to_list()
+
+                    if len(date_time_list_list) > 0:
+                        date_time_intersect = (set(date_time_list_list).intersection(date_time_list))
+                        if len(date_time_intersect) == 0:
+                            df_list.append(df)
+                    else:
+                        df_list.append(df)
+
+                    date_time_list_list += date_time_list
+
             cols = ['datetime', 'date', 'time', 'open', 'high', 'low', 'close', 'volume', 'expiry_date','RTH']
             df = pd.concat(df_list)
-    
-            df = df[df['date'] >= start_date]
-            df = df[df['date'] <= end_date]
-            df['date'] = df['date'].astype(str)
-            df['time'] = df['time'].astype(str)
-            df['time'] = df['time'].str.zfill(6)
-            df['datetime'] = df['date'] + ' ' + df['time']
+
             df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
             df['RTH'] = df['RTH'].astype(bool)
             
             df = df[cols]
             df = df.set_index(keys='datetime')
             df = df.sort_index(ascending=True)
-    
+
             return df
     
         else:
@@ -390,7 +411,7 @@ class Database:
                     err_msg += error + ','
             print(err_msg)
 
-    def get_hk_buyback_by_code(self, code, agg_value):
+    def get_hk_buyback_by_code(self, code, agg_value = False):
 
         link_url = 'http://www.hkfdb.net/data_api?'
         token_str = 'token=' + str(self.authToken)
@@ -472,6 +493,7 @@ class Database:
         df = df.sort_index()
 
         return df
+
     def get_hk_earning_calendar_by_date(self, start_date, end_date):
 
         check_bool_dict = self.check_start_end_date(start_date, end_date)
@@ -904,8 +926,8 @@ class Database:
         link_str = link_url + '&' + token_str + '&' + database_str
         
         response = requests.get(link_str)
-        #result = json.loads(json.loads(response.content).replace("'", "\""))
-        result = ast.literal_eval(json.loads(response.content))
+        json1 = json.loads(response.content).replace("'", "\"").replace('nan','NaN')
+        result = json.loads(json1)
 
         df = pd.DataFrame(result)
         df['ipo_date'] = pd.to_datetime(df['ipo_date'],format='%Y-%m-%d')
@@ -919,8 +941,7 @@ class Database:
         
         df['share_issued'] = df['share_issued'].astype(int)
         df['market_capital'] = df['market_capital'].astype(int)
-        
-        
+
         return df
 
     def get_hk_ipo_hist(self):
@@ -1653,7 +1674,7 @@ def get_holiday_from_gov(year):
 
     return holiday_date_list, half_day_mkt_date_list
 
-def get_hk_holiday_and_expiry_date(start_year, end_year, format='int'):
+def get_hk_holiday_and_expiry_date(start_year, end_year, format='int', str_format = '%Y%m%d'):
     holiday_date_list = []
     half_day_mkt_date_list = []
 
@@ -1679,32 +1700,32 @@ def get_hk_holiday_and_expiry_date(start_year, end_year, format='int'):
                 if test_date.weekday() < 5 and test_date not in holiday_date_list:
                     trading_days += 1
                 if trading_days == 2:
-                    if format == 'str':
-                        expiry_date = test_date.strftime('%Y%m%d')
-                    elif format == 'int':
+                    if format == 'str' or format == 'string':
+                        expiry_date = test_date.strftime(str_format)
+                    elif format == 'int' or format == 'integer':
                         expiry_date = int(test_date.strftime('%Y%m%d'))
-                    elif format == 'datetime':
+                    elif format == 'dt' or format == 'datetime':
                         expiry_date = test_date
                     expiry_date_list.append(expiry_date)
                     break
 
     holiday_list = []
     for day in holiday_date_list:
-        if format == 'str':
-            holiday = day.strftime('%Y%m%d')
-        elif format == 'int':
+        if format == 'str' or format == 'string':
+            holiday = day.strftime(str_format)
+        elif format == 'int' or format == 'integer':
             holiday = int(day.strftime('%Y%m%d'))
-        elif format == 'datetime':
+        elif format == 'dt' or format == 'datetime':
             holiday = day
         holiday_list.append(holiday)
 
     half_day_mkt_list = []
     for day in half_day_mkt_date_list:
-        if format == 'str':
-            holiday = day.strftime('%Y%m%d')
-        elif format == 'int':
+        if format == 'str' or format == 'string':
+            holiday = day.strftime(str_format)
+        elif format == 'int' or format == 'integer':
             holiday = int(day.strftime('%Y%m%d'))
-        elif format == 'datetime':
+        elif format == 'dt' or format == 'datetime':
             holiday = day
         half_day_mkt_list.append(holiday)
 
